@@ -20,21 +20,25 @@
 
 
 module APRS4R
-  APRS4RVersion = [0, 9, 1].join( '.')
+  APRS4RVersion = [0, 9, 2].join( '.')
+
+  APRS4RCall = "AP4R0"
 end
 
-path = ENV['APRS4R_ETC']
-path = "/etc/aprs4r" unless ENV['APRS4R_ETC']
 
-log4r_file = ENV['APRS4R_LOG4R']
-log4r_file = "log4r.yaml" unless ENV['APRS4R_LOG4R']
 
-print "log4r: #{path}/#{log4r_file}\n"
+path = ENV['APRS4R_ETC'] || "/etc/aprs4r" 
+log4r = ENV['APRS4R_LOG4R'] || "log4r.yaml"
+file = ARGV[0] || "#{path}/default.yaml"
+
+print "path: #{path}\n"
+print "log4r: #{log4r}\n"
+print "file: #{file}\n"
 
 require 'Logger'
 require 'Log4rConfigurator'
 
-Log4rConfigurator::configure_from_yaml_file( "#{path}/#{log4r_file}")
+Log4rConfigurator::configure_from_yaml_file( "#{path}/#{log4r}")
 
 require 'aprs4r/APRS4RConfiguration'
 
@@ -45,16 +49,10 @@ require 'aprs4r/GatewayManager'
 require 'aprs4r/PluginManager'
 
 
-# load aprs4r config
-file = "#{path}/default.yaml"
-if ARGV.length == 1 
-  file = ARGV[0]
-end
-
 # we want to see stacktraces from all threads
 Thread.abort_on_exception = true
 
-configuration = APRS4RConfiguration.loadConfiguration( file)
+configuration = APRS4RConfiguration.loadConfiguration( "#{file}")
 
 # init aprs4r core module
 SocketManager.init( configuration)
