@@ -36,6 +36,9 @@ require 'aprs4r/DigipeaterConfiguration'
 require 'aprs4r/PluginConfiguration'
 require 'aprs4r/LCD4LinuxPluginConfiguration'
 require 'aprs4r/XMLDataPluginConfiguration'
+require 'aprs4r/QueryPluginConfiguration'
+require 'aprs4r/MessagePluginConfiguration'
+require 'aprs4r/ISServerConfiguration'
 
 
 class APRS4RConfiguration
