@@ -34,60 +34,62 @@ class ISSocket
   @@logger = Logger.get_logger( "ISSocket")
 
 
-  def ISSocket.generatePassword( value)
-    @@logger.info( "generatePassword( value)")
-
-    callsign = APRSCall.parse( value)
-    call = callsign.call
-
-    if call.nil?
-      return 
-    end
-
-    # init value
-    value = 0x73E2
-    i = 0
-
-    while i < call.length
-      value = value ^ (call[i].to_i << 8)
-      value = value ^ (call[i+1].to_i)
-
-      i += 2
-    end
-
-    return (value & 0x7FFF).to_s
-  end
-
-
-  def initialize( configuration)
-    @@logger.info( "initialize( configuration)")
-
-    @name = configuration.name
-    @type = configuration.type
-    @enable = configuration.enable
-
-    @host = configuration.host
-    @port = configuration.port
-    @username = configuration.username
-    @password = ISSocket.generatePassword( @username)
-    @filter = configuration.filter
-    @timeout = configuration.timeout
-    @duplicatePeriod = configuration.duplicatePeriod
+  def initialize( configuration, socket = nil)
+    @@logger.info( "initialize( configuration | socket)")
 
     @mutex = Mutex.new
 
-    if @enable 
-      initISSocket()
+    @@logger.debug( "configuration: #{configuration}")
+
+    if configuration 
+      # socket client mode 
+      @clientMode = true 
+      
+      @name = configuration.name
+      @type = configuration.type
+      @enable = configuration.enable
+      
+      @host = configuration.host
+      @port = configuration.port
+      @username = configuration.username
+      @password = ISSocket.generatePassword( @username)
+      @filter = configuration.filter
+      @timeout = configuration.timeout
+      @duplicatePeriod = configuration.duplicatePeriod
+    else 
+      # socket server mode 
+      @clientMode = false
+
+      @socket = socket
+      @socket.nonblock = true
+      @socket.sync = true 
     end
-    
+
+    initISSocket()
+
     return
   end
-  
+
 
   def initISSocket()
-    @@logger.info( "initISSocket")
+    @@logger.info( "initISSocket()")
 
-    @socket = nil
+    if ! @enable 
+      return
+    end
+
+    if @clientMode
+      initClientISSocket
+    else 
+      initServerISSocket
+    end
+
+    return
+  end
+
+
+  def initClientISSocket()
+    @@logger.info( "initClientISSocket()")
 
     begin 
       @socket = TCPSocket.new( @host, @port)
@@ -104,15 +106,35 @@ class ISSocket
       @socket.puts( @filter)
     rescue Exception => ex
       @@logger.warn( "initISSocket::ex: #{ex}")
+
+      # wait 5 seconds before next try
+      sleep 5
+      retry
     end
 
     return
   end
 
 
+  def initServerISSocket()
+    @@logger.info( "initServerISSocket()")
+
+    software = "\# aprs4r #{APRS4R::APRS4RVersion}"
+    @socket.puts( software)
+
+    line = @socket.gets
+    line.chomp! if line
+
+    @@logger.debug( "start: #{line}")
+
+    return
+  end
+
+
   def reinitISSocket()
     @@logger.info( "reinitISSocket")
 
+    # close socket (with timeout)
     begin
       Timeout::timeout( 5) do
         if ! @socket.nil?
@@ -122,32 +144,91 @@ class ISSocket
     rescue Exception => ex
       @@logger.warn( "reinitISSocket::ex: #{ex}")
     end
-
-    @socket = nil
-
-    if @enable
-      initISSocket()
+    
+    if @clientMode
+      reinitClientISSocket()
+    else 
+      reinitServerISSocket()
     end
 
     return
   end
+  
+  
+  def reinitClientISSocket()
+    @@logger.info( "reinitClientISSocket")
+
+    initClientISSocket()
+    
+    return
+  end
+
+
+  def reinitServerISSocket()
+    @@logger.info( "reinitServerISSocket")
+
+    raise "client side down"
+
+    return
+  end
 
 
   def readAPRSMessage
     @@logger.info( "readAPRSMessage")
 
-    message = APRSMessage.new
-
     data = readData
 
+    message = parseAPRSMessage( data)
+
+    @@logger.debug( "readAPRSMessage: message: #{message} #{message.nil?}")
+
+    return message
+  end
+
+  
+  def readData
+    @@logger.info( "readData")
+
+    line = nil
+
+    begin
+      Timeout::timeout( @timeout) do
+
+        if @socket
+          line = @socket.gets
+        end
+        
+        # if nil, an exception is raised (and reinit is started)
+        line.chomp!
+        @@logger.debug( "readData::line: -#{line}-")
+
+      end
+    rescue Exception => ex
+      @@logger.warn( "readData::ex: " + ex)
+      reinitISSocket()
+    end until line !~ /^#/
+    
+    return line
+  end
+  
+
+  def parseAPRSMessage( data)
+    @@logger.info( "parseAPRSMessage( data)")
+    
+    @@logger.debug( "data: #{data}")
+
     if data.nil?
       return nil
     end
+    
+    # todo more robust message parsing 
 
+    message = nil
     headerIndex = data.index( ':')
     
     if !headerIndex.nil?
         
+      message = APRSMessage.new
       # header
       header = data[0...headerIndex]
       
@@ -173,7 +254,6 @@ class ISSocket
       body = data[headerIndex+1...data.length]
       message.payload = body
     end
-    
     return message
   end
 
@@ -181,13 +261,24 @@ class ISSocket
   def writeAPRSMessage( message) 
     @@logger.info( "writeAPRSMessage( message)")
   
+    data = serializeAPRSMessage( message)
+
+    writeData( data)
+    
+    return
+  end
+
+  
+  def serializeAPRSMessage( message)
+    @@logger.info( "serializeAPRSMessage( message)")
+
     if message.nil?
-    	return
+      return nil
     end
     
     if message.source.nil? || message.destination.nil? 
       @@logger.warn( "NIL\n")
-      return
+      return nil
     end
 
     data = String.new
@@ -202,76 +293,55 @@ class ISSocket
 
     data = data + ":" + message.payload
 
-    writeData( data)
-    
-    return
+    return data
   end
 
 
-  def readData
-    @@logger.info( "readData")
-
-    @mutex.synchronize do 
-      if @socket.nil?
-        reinitISSocket()
-      end
-    end
-
-    line = nil
-
-    begin
-      Timeout::timeout( @timeout) do
-
-        if ! @socket.nil?
-          line = @socket.gets
-        end
-        
-        if !line.nil?
-          line = line.chomp
-        end
-
-        @@logger.debug( "readData::line: -#{line}-")
-
-      end
-    rescue Exception => ex
-      @@logger.warn( "readData::ex: " + ex)
-      reinitISSocket()
-      # FIXME 
-      return nil
-    end until line !~ /^#/
-    
-    return line
-  end
-  
-
   def writeData( data)
     @@logger.info( "writeData( data)")
 
-    @mutex.synchronize do 
-      if @socket.nil?
-        reinitISSocket()
-      end
+    if data.nil?
+      return
     end
 
     @mutex.synchronize do
       begin
-        if ! @socket.nil?
-          @@logger.debug( "writeData::data: -#{data}-")
-          @socket.puts( data)
-          @socket.flush
-        end
+        @@logger.debug( "writeData::data: -#{data}-")
+        @socket.puts( data)
+        @socket.flush
       rescue Exception => ex
-
         @@logger.warn( "ex: " + ex)
         reinitISSocket()
-        
-        if ! @socket.nil?
-          @socket.puts( data)
-        end
       end
     end
 
     return
   end
+
   
+  def ISSocket.generatePassword( value)
+    @@logger.info( "generatePassword( value)")
+
+    callsign = APRSCall.parse( value)
+    call = callsign.call
+
+    if call.nil?
+      return 
+    end
+
+    # init value
+    value = 0x73E2
+    i = 0
+
+    while i < call.length
+      value = value ^ (call[i].to_i << 8)
+      value = value ^ (call[i+1].to_i)
+
+      i += 2
+    end
+
+    return (value & 0x7FFF).to_s
+  end
+
+
 end
