@@ -35,13 +35,14 @@ class KISSDevice
   @@logger = Logger.get_logger( "KISSDevice")
   
 
-  def initialize( device, call, speed, timeout)
-    @@logger.info( "initialize( device)")
+  def initialize( device, call, speed, timeout, mode)
+    @@logger.info( "initialize( device, call, speed, timeout, mode)")
     
     @device = device
     @call = call
     @speed = speed
     @timeout = timeout
+    @mode = mode
 
     @mutex = Mutex.new
     
@@ -65,42 +66,17 @@ class KISSDevice
       @port.sync = true
       
       @port.flow_control = SerialPort::SOFT
+
+      if mode.index( "tnc2/northlink") != nil
+        initTNC2Northlink
+      elsif mode.index( "tnc2/tapr") != nil
+        initTNC2TAPR
+      elsif mode.index( "kenwood") != nil
+        initKenwood
+      else
+        initDefault
+      end
       
-      # send exit frame 
-      # data = String.new
-      
-      # data << @@FEND
-      # data << 0xFF
-      # data << @@FEND
-      # data << "\r" 
-
-      # @@logger.debug( "send KISS exit frame");
-      # @port.write( data)
-      # sleep 1
-      
-      @@logger.debug( "restart kenwood tnc")
-      @port.write( "TC 1\r")
-      sleep 2
-      @port.write( "TC 0\r")
-      sleep 2
-      
-
-      # enable kiss mode
-      @@logger.debug( "send: mycall #{@call}");
-      @port.write( "mycall #{@call}\r")
-      sleep 1
-
-      @@logger.debug( "send: hbaud #{@speed}");
-      @port.write( "hbaud #{@speed}\r");
-      sleep 1
-
-      @@logger.debug( "send: kiss on ");
-      @port.write( "kiss on\r");
-      sleep 1
-
-      @@logger.debug( "send: restart");
-      @port.write( "restart\r")
-      sleep 1
     end
 
     return
@@ -116,6 +92,138 @@ class KISSDevice
   end
 
 
+  def initTNC2Northlink
+    @@logger.info( "initTNC2Northlink")
+
+    # send exit frame 
+    data = String.new
+      
+    data << @@FEND
+    data << 0xFF
+    data << @@FEND
+    data << "\r" 
+
+    @@logger.debug( "send KISS exit frame");
+    @port.write( data)
+    sleep 1
+
+    data = String.new
+    data << ?\C-X
+    data << ?\C-[
+    data << "@K"
+    data << "\r"
+
+    @@logger.debug( "send KISS init sequence")
+    @port.write( data)
+    sleep 1
+
+    return
+  end
+
+
+  def initTNC2TAPR
+    @@logger.info( "initTNC2TAPR")
+    
+    # send exit frame 
+    data = String.new
+      
+    data << @@FEND
+    data << 0xFF
+    data << @@FEND
+    data << "\r" 
+
+    @@logger.debug( "send KISS exit frame");
+    @port.write( data)
+    sleep 1
+      
+    # enable kiss mode
+    # @@logger.debug( "send: mycall #{@call}");
+    # @port.write( "mycall #{@call}\r")
+    # sleep 1
+
+    # @@logger.debug( "send: hbaud #{@speed}");
+    # @port.write( "hbaud #{@speed}\r");
+    # sleep 1
+
+    @@logger.debug( "send: kiss on ");
+    @port.write( "kiss on\r");
+    sleep 1
+
+    @@logger.debug( "send: restart");
+    @port.write( "restart\r")
+    sleep 1
+
+    return
+  end
+
+
+  def initKenwood
+    @@logger.info( "initKenwood")
+
+    @@logger.debug( "restart kenwood tnc")
+    @port.write( "TC 1\r")
+    sleep 2
+    @port.write( "TC 0\r")
+    sleep 2
+      
+
+    # enable kiss mode
+    @@logger.debug( "send: mycall #{@call}");
+    @port.write( "mycall #{@call}\r")
+    sleep 1
+    
+    @@logger.debug( "send: hbaud #{@speed}");
+    @port.write( "hbaud #{@speed}\r");
+    sleep 1
+    
+    @@logger.debug( "send: kiss on ");
+    @port.write( "kiss on\r");
+    sleep 1
+    
+    @@logger.debug( "send: restart");
+    @port.write( "restart\r")
+    sleep 1
+    
+    return
+  end
+
+
+  def initDefault
+    @@logger.info( "initDefault")
+    
+    # send exit frame 
+    data = String.new
+      
+    data << @@FEND
+    data << 0xFF
+    data << @@FEND
+    data << "\r" 
+
+    @@logger.debug( "send KISS exit frame");
+    @port.write( data)
+    sleep 1
+      
+    # enable kiss mode
+    # @@logger.debug( "send: mycall #{@call}");
+    # @port.write( "mycall #{@call}\r")
+    # sleep 1
+
+    # @@logger.debug( "send: hbaud #{@speed}");
+    # @port.write( "hbaud #{@speed}\r");
+    # sleep 1
+
+    @@logger.debug( "send: kiss on ");
+    @port.write( "kiss on\r");
+    sleep 1
+
+    @@logger.debug( "send: restart");
+    @port.write( "restart\r")
+    sleep 1
+
+    return
+  end
+
+
   def readFrame
     @@logger.info( "readFrame")
 
